local ADDON_NAME, namespace = ...
local L = namespace.L

--+ types:						
--+---+-----------------------			 
--+ a + achievement						
--+ t + toy
--+ m + transMog
--+ p + pet
--+ n + mouNt
--+ i + item
--+ h + headline
--+---+-------------------【ツ】
local topper = false
local popper = false
local PlayerGUID = UnitGUID("player")
local _, EngClass, _ = UnitClass("player")
local PlayerName, _ = UnitName("player") 
local PlayerID = PlayerName..'#'..EngClass
local optionsFrame
local LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN
local LOOT_ITEM_BONUS_ROLL_PATTERN
local LOOT_ITEM_CREATED_PATTERN
local LOOT_ITEM_CREATED_MULTIPLE_PATTERN 
local LOOT_ITEM_PATTERN
local LOOT_ITEM_MULTIPLE_PATTERN
local LOOT_ITEM_PUSHED_PATTERN
local LOOT_ITEM_PUSHED_MULTIPLE_PATTERN

local LOOT_ITEM_OTHER_PATTERN
local LOOT_ITEM_MULTIPLE_OTHER_PATTERN
local LOOT_ITEM_PUSHED_OTHER_PATTERN	
local LOOT_ITEM_PUSHED_MULTIPLE_OTHER_PATTERN
local LOOT_ITEM_BONUS_ROLL_OTHER_PATTERN
local LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN
local refresh_requested = false

local C_ToyBox_GetToyInfo = C_ToyBox.GetToyInfo
local C_ToyBox_GetToyFromIndex = C_ToyBox.GetToyFromIndex
local C_PetJournal_GetPetInfoByPetID=C_PetJournal.GetPetInfoByPetID
local C_PetJournal_GetPetInfoBySpeciesID=C_PetJournal.GetPetInfoBySpeciesID
local C_TransmogCollection_GetAppearanceSourceInfo = C_TransmogCollection.GetAppearanceSourceInfo
local C_PetJournal_GetBattlePetLink = C_PetJournal.GetBattlePetLink

Recent = LibStub("AceAddon-3.0"):NewAddon("Recent", "AceConsole-3.0", "AceEvent-3.0")

RecentLDB = LibStub("LibDataBroker-1.1"):NewDataObject("Recent", {
    type = "data",
    text = "Recent",
    icon = "Interface\\Icons\\INV_Inscription_Trinket02.blp",
	OnClick = function(_, button) 
		if button== "LeftButton" then
			if RecentListFrame:IsShown() then 
				RecentListFrame:Hide() 
			else 
				RecentListFrame:Show() 
			end 
		end
		if button=="RightButton" then
			if optionsFrame then
				if InterfaceOptionsFrame:IsShown() then
					InterfaceOptionsFrame:Hide();
				else
					InterfaceOptionsFrame_OpenToCategory(optionsFrame)
					InterfaceOptionsFrame_OpenToCategory(optionsFrame)
				end
			end
		end
	end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine("Recent")
		tooltip:AddLine(L["|cFFff4040<Left Click>|r to open log"], 0.2, 1, 0.2)
		tooltip:AddLine(L["|cFFff4040<Right Click>|r to open options"], 0.2, 1, 0.2)
	end,
})
local icon = LibStub("LibDBIcon-1.0")

local options = {
    name = "Recent",
    handler = Recent,
    type = "group",
    args = {
		section1 = {
			name = AUCTION_DURATION,
			type = "group",
			inline = true,
			order = 0,
			args = {
				Note = {
					name = "  ",
					type = "description",
					order = 6,
				},     
				days = {
					type = 'range',
					name = L["global storage period"],
					desc = L["number of days to keep all the informations"],
					min = 1,
					softMax = 10,
					max = 10,
					step = 1,
					order = 10,
					get = function() return Recent.db.profile.threshold end,
					set = function(info, value)
						Recent.db.profile.threshold = value
					end,
					width = "normal",
				},  
				Note1 = {
					name = DISABLED_FONT_COLOR_CODE  .. L["     Only days with records count towards this limit.\n     And obviously: More days need more memory!"],
					type = "description",
					order = 20,
					width= "double"
				},   
				Note3 = {
					name = "  ",
					type = "description",
					order = 22,
				},    		
			},	
		},		
		section2 = {
			name = LOOT,
			type = "group",
			inline = true,
			order = 30,
			args = {
				Note6 = {
					name = "  ",
					type = "description",
					order = 31,
				},    				    
				logPartyLoot = {
					type = "toggle",
					name = L["log party loot"],
					desc = L["also logs loot of other players when in party or raid"],
					get = "GetOption",
					set = "SetOption",
					order = 32,
					width = "full"
				},
				Note0 = {
					name = DISABLED_FONT_COLOR_CODE .. L["       Remark: This will take additional memory."],
					type = "description",
					order = 33,
					width = "full"
				},
				Note5 = {
					name = "  ",
					type = "description",
					order = 35,
				},    				    
			},
		},					
		section3 = {
			name = "Minimap",
			type = "group",
			inline = true,
			order = 40,
			args = {
				Note2 = {
					name = "  ",
					type = "description",
					order = 46,
				},     
				minimap = {
					type = "toggle",	
					name = L["show minimap button"],
					desc = L["show or hide that little icon on your minimap"],
					get = function() return not Recent.db.profile.minimap.hide end,
					set = function(info, value)
						value = not value
						Recent.db.profile.minimap.hide = value
						if (value) then
							icon:Hide("Recent")
						else
							icon:Show("Recent")
						end
					end,
					order=60,
					width = "full"
				},	
				Note4 = {
					name = "  ",
					type = "description",
					order = 76,
				},     
			},     
		},		
		section4 = {
			name = "Auto Popup",
			type = "group",
			inline = true,
			order = 80,
			args = {
				Note8 = {
					name = "  ",
					type = "description",
					order = 86,
				},     
				popup = {
					type = "toggle",	
					name = L["show Recent if something is logged"],
					desc = L["show Recent if something is logged"],
					get = function() return Recent.db.profile.popup end,
					set = function(info, value)
						Recent.db.profile.popup = value
					end,
					order=90,
					width = "full"
				},	
				Note10 = {
					name = DISABLED_FONT_COLOR_CODE .. L["     Your selected filters also apply here.\n     Recent won't show up until you're out of combat!"],
					type = "description",
					order = 95,
					width= "full"
				},   
				Note9 = {
					name = "  ",
					type = "description",
					order = 96,
				},     
			},     
		},		
    },
}

local defaults = {
		profile = {
			minimap = {
				hide = false,
			},
			showPartyLoot = false,
			showJunk = false,
			logPartyLoot = false,
			threshold = 7,
			showAch = true,	
			showTrans = true,
			showPets = true,
			showToys = true,
			showMounts = true,
			popup = false,
		},
		char = {
			history = {}
		} 
	}

function Recent:OnInitialize() ----------------------------------------------------------------------------------
	self.db = LibStub("AceDB-3.0"):New("RecentPrefsDB", defaults)
	icon:Register("Recent", RecentLDB, self.db.profile.minimap)

--	RecentPrefsDB.message = "Welcome Home!"
	self:RegisterChatCommand("rct", "OnChatCommand")
	self:RegisterChatCommand("recent", "OnChatCommand")

    LibStub("AceConfig-3.0"):RegisterOptionsTable("Recent", options)
    optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Recent", "Recent")

	RecentDB = self.db.char.history

	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("CRITERIA_EARNED")
	self:RegisterEvent("LOOT_ITEM_ROLL_WON")
	self:RegisterEvent("TRANSMOG_COLLECTION_SOURCE_ADDED")
	self:RegisterEvent("NEW_PET_ADDED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")	
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	RecentListFrame.ListScrollFrame.update = RecentUpdateList;

	LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN =LOOT_ITEM_BONUS_ROLL_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_BONUS_ROLL_PATTERN = LOOT_ITEM_BONUS_ROLL_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_CREATED_PATTERN = LOOT_ITEM_CREATED_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_CREATED_MULTIPLE_PATTERN = LOOT_ITEM_CREATED_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_PATTERN = LOOT_ITEM_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_MULTIPLE_PATTERN = LOOT_ITEM_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_PUSHED_PATTERN = LOOT_ITEM_PUSHED_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_PUSHED_MULTIPLE_PATTERN = LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
		
	LOOT_ITEM_OTHER_PATTERN=LOOT_ITEM:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_MULTIPLE_OTHER_PATTERN=LOOT_ITEM_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_PUSHED_OTHER_PATTERN=LOOT_ITEM_PUSHED:gsub("%%s", "(.+)"):gsub("^", "^")	
	LOOT_ITEM_PUSHED_MULTIPLE_OTHER_PATTERN=LOOT_ITEM_PUSHED_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_BONUS_ROLL_OTHER_PATTERN=LOOT_ITEM_BONUS_ROLL:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_BONUS_ROLL_MULTIPLE_OTHER_PATTERN=LOOT_ITEM_BONUS_ROLL_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")

	local temp=""
	local ctr=0
	local threshold= tonumber(Recent.db.profile.threshold)                                   --   DAYS 2 KEEP

	for v, p in pairs(RecentDB) do
	   if p.sd ~= temp then
		  temp=p.sd
		  ctr=ctr+1
	   end   
	end   

	temp=""												-- remove older entries
	while ctr>threshold do
	   
	   if temp == "" then temp=RecentDB[1].sd end 
	   if temp == RecentDB[1].sd then
			table.remove(RecentDB,1)
	   else     
		  ctr=ctr-1
		  temp=""
	   end
	end

	Recent:NewDay()
	Recent:RecentCreateFilteredList(true)
	RDB=RecentFiltered
	HybridScrollFrame_CreateButtons(RecentListFrame.ListScrollFrame, "RecentListButtonTemplate", 46, 0)   

	local _, _, _, wowtocversion = GetBuildInfo()
	if (wowtocversion > 90000) then Mixin(RecentListFrame, BackdropTemplateMixin) end
	RecentListFrame:SetBackdrop({ bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 32,	edgeSize = 32, insets = {left = 12, right = 12, top = 12, bottom = 11}})

	local majorVersion,minorVersion = (GetAddOnMetadata("ElvUI","Version") or ""):match("^(%d+)%.(%d+)")
	--print(majorVersion,minorVersion)
	
	if majorVersion then
	   
	   local E, L, P, G = unpack(ElvUI)
	   local S = E:GetModule("Skins")
	   
	   local r, g, b = unpack(isUnitFrameElement and E.media.unitframeBorderColor or E.media.bordercolor)
	   RecentListFrame:SetBackdrop({edgeFile="Interface\\ChatFrame\\ChatFrameBackground",edgeSize=4})
	   RecentListFrame:SetBackdropBorderColor(r,g,b)
	   RecentListFrame:SetTemplate("Transparent")
	   RecentListFrame.LeftInset:StripTextures()
	   
	   RecentListFrame.BottomLeftInset:StripTextures()
	   
	   RecentListFrame.BottomLeftInset:CreateShadow(2)
	   RecentListFrame.LeftInset:CreateShadow(2)
	   
	   local borderr, borderg, borderb = unpack(isUnitFrameElement and E.media.unitframeBorderColor or E.media.bordercolor)
	   RecentListFrame.BottomLeftInset.shadow:SetBackdropColor(backdropr, backdropg, backdropb, 0)
	   RecentListFrame.BottomLeftInset.shadow:SetBackdropBorderColor(borderr, borderg, borderb, 0.9)
	   RecentListFrame.LeftInset.shadow:SetBackdropColor(backdropr, backdropg, backdropb, 0)
	   RecentListFrame.LeftInset.shadow:SetBackdropBorderColor(borderr, borderg, borderb, 0.9)
	   
	   S:HandleCloseButton(RecentListFrame.CloseButton,nil,"")
	   S:HandleButton(RecentListFrameFilterButton)
	   S:HandleNextPrevButton(RecentListFrameListScrollFrameScrollBarScrollUpButton) 
	   S:HandleNextPrevButton(RecentListFrameListScrollFrameScrollBarScrollDownButton) 
	   --S:HandleDropDownBox(RecentListFrameFilterButton)
	   local scrollBar = RecentListFrameListScrollFrameScrollBar
	   scrollBar:StripTextures()
	   scrollBar:GetThumbTexture():SetTexture(nil)
	   scrollBar:SetThumbTexture("Interface\\AddOns\\ElvUI\\media\\textures\\minimalist")
	   scrollBar:GetThumbTexture():SetVertexColor(1,1,1)
	   scrollBar:GetThumbTexture():Size(14,14)
	end

	if C_ToyBox.GetNumFilteredToys() < 555 then
		C_Timer.After(5, function() RecentUpdateList() end)
	else
		RecentUpdateList()
	end
end

function Recent:getPet(lonk) ----------------------------------------------------------------------------------
	local linkType, linkID, battlePetID, itemRarity
	local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType, missinginfos
	
	_, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",lonk)
	if linkType=="battlepet" then
	   speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal_GetPetInfoByPetID(battlePetID); 
	   if speciesID == nil then petName, petIcon, petType = C_PetJournal_GetPetInfoBySpeciesID(linkID) end
	   _, _, level, itemRarity, _, _, _, battlePetID = strsplit(":", lonk)
	   
	   if (itemRarity==nil) then
		  missinginfos = 1
		  itemRarity = 1
		  level = 0
		  _ , petName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
	   end   
	   return level, petIcon, petName, itemRarity, missinginfos
	end
end

function RecentRefreshList() ----------------------------------------------------------------------------------
	Recent:RecentCreateFilteredList(true)
	RDB=RecentFiltered
	if not InCombatLockdown() and topper == true then
		topper=false
		RecentListFrame.ListScrollFrame.scrollBar:SetValue(1)
	end
	RecentUpdateList()
end

function Recent:UnitInOurParty(who) ----------------------------------------------------------------------------------
	if who == PlayerGUID then 
		return true
	else
		local _, _, _, _, _, name, realm = GetPlayerInfoByGUID(who)
		if UnitInParty(name) then 
			return true
		else
			return false
		end
	end
end

function Recent:CHAT_MSG_LOOT(event, ...)	-----------------------------------------------------

	local message, _, _, _, _, _, _, _, _, _, _, guid = ...
	if guid==PlayerGUID or (Recent:UnitInOurParty(guid)==true and Recent.db.profile.logPartyLoot==true) then
--	if Recent:UnitInOurParty(guid)==true then
		local link, quantity = message:match(LOOT_ITEM_MULTIPLE_PATTERN)
		if not link then
			link, quantity = message:match(LOOT_ITEM_PUSHED_MULTIPLE_PATTERN)
			if not link then
			   link, quantity = message:match(LOOT_ITEM_CREATED_MULTIPLE_PATTERN)
			   if not link then
				  link, quantity = message:match(LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN)
				  if not link then
					 quantity, link = 1, message:match(LOOT_ITEM_PATTERN)
					 if not link then
						quantity, link = 1, message:match(LOOT_ITEM_PUSHED_PATTERN)
						if not link then
						   quantity, link = 1, message:match(LOOT_ITEM_CREATED_PATTERN)
						   if not link then
							  quantity, link = 1, message:match(LOOT_ITEM_BONUS_ROLL_PATTERN)
							  if not link then
								 quantity, _, link = 1, message:match(LOOT_ITEM_OTHER_PATTERN)
								 if not link then
									quantity, _, link = 1, message:match(LOOT_ITEM_PUSHED_OTHER_PATTERN)
									if not link then
									   quantity,  _,  link = 1, message:match(LOOT_ITEM_BONUS_ROLL_OTHER_PATTERN)
									   if not link then
										  _, link, quantity = message:match(LOOT_ITEM_MULTIPLE_OTHER_PATTERN)
										  if not link then
											 _, link, quantity  = message:match(LOOT_ITEM_PUSHED_MULTIPLE_OTHER_PATTERN)
											 if not link then
												_, link, quantity  = message:match(LOOT_ITEM_BONUS_ROLL_MULTIPLE_OTHER_PATTERN)
											 end                                                                                    
										  end
									   end
									end
								 end
							  end
						   end
						end
					 end
				  end        
			   end
			end
		 end

		if link then
			local _, eclass, _, _, _, pname, realm = GetPlayerInfoByGUID(guid)
			local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture, itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(link)
			local a, color, b, linkType, linkID = strsplit(":|H", link)		
			if linkID~= nil then 
				local itemID, toyName, icon, isFavorite, hasFanfare = C_ToyBox_GetToyInfo(linkID)
				local itype= "i"

				if (itemID ~= nil)  then itype="t" 
				elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_COMPANION_PET) then itype="p"
				elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_MOUNT) then itype="n"
				end

				if Recent:sameAsBefore(link, pname .. "#"..eclass) then
					local max = table.getn(RecentDB)
					local cnt
					if RecentDB[max].qnty == nil then cnt =1 else cnt=RecentDB[max].qnty end
					RecentDB[max].qnty = cnt + quantity
					topper=true
					if refresh_requested == false then
						refresh_requested=true
						C_Timer.After(1, RecentRefreshList)
					end    
				else
					if itemRarity~= nil then
						if itemRarity > 2 or itype=="p" or itype=="n" or itype=="t" then 
							Recent:NewDay()
							table.insert(RecentDB, {ts=time() , lnk=link ,qnty= quantity , sd=Recent:sortDate(time()), loc= (string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=itemID, type=itype, addinfo=nil, player=pname .. "#"..eclass })  
							topper=true
							if refresh_requested == false then
								refresh_requested=true
								C_Timer.After(1, RecentRefreshList)
							end
						end   
					end
				end
			end
		end
	end
end

function Recent:ACHIEVEMENT_EARNED(eventName, ...)  -----------------------------------------
	local achID, already = ...
	if achID then 
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(achID)

		local link = GetAchievementLink(achID)
		Recent:NewDay()

		table.insert(RecentDB, {ts=time() , lnk=link ,qnty=nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=achID, type="a", addinfo=nil})  

		topper=true
		if refresh_requested == false then
			refresh_requested=true
			C_Timer.After(1, RecentRefreshList)
		end
	end
end

function Recent:CRITERIA_EARNED(eventName, ...) -------------------------------------------
	local achID, criteriaString  = ...
	if achID then
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(achID)
		if completed == false then
			local link = GetAchievementLink(achID)
			Recent:NewDay()

			table.insert(RecentDB, {ts=time() , lnk=link ,qnty=nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=achID, type="a", addinfo=criteriaString})  
			topper=true
			if refresh_requested == false then
				refresh_requested=true
				C_Timer.After(1, RecentRefreshList)
			end
		end
	end 
end

function Recent:LOOT_ITEM_ROLL_WON(eventName, ...)  ----------------------------------------

	local link, quantity, rollType, roll, isUpgraded = ...
	local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType,
		itemSubType, itemStackCount, itemEquipLoc, texture,
		itemSellPrice, itemClassID, itemSubClassID, bindType, expacID,
		itemSetID, isCraftingReagent = GetItemInfo(link)

	local a, color, b, linkType, linkID = strsplit(":|H", RecentDB[self:GetParent().index].lnk);
	if linkID then				
		local itemID, toyName, icon, isFavorite, hasFanfare = C_ToyBox_GetToyInfo(linkID)

		local itype= "i"
		if itemID then itype="t" 
		elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_COMPANION_PET) then itype="p"
		elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_MOUNT) then itype="n"
		end
		
		if itemRarity > 2 or itype=="p" or itype=="n" or itype=="t" then		
			Recent:NewDay()
			table.insert(RecentDB, {ts=time() , lnk=link ,qnty= quantity , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) ,  iid=itemID, type=itype, addinfo=nil})  
			topper=true
			if refresh_requested == false then
				refresh_requested=true
				C_Timer.After(1, RecentRefreshList)
			end
		end   
	end
end

function Recent:TRANSMOG_COLLECTION_SOURCE_ADDED(eventName, ...)  ---------------------------------------
	sourceID, isAdded, attempt = ...
	if sourceID then
		local _, visualID, _, icon, _, _, link = C_TransmogCollection_GetAppearanceSourceInfo(sourceID)

		if string.len(link)>0 then
			Recent:NewDay()
			table.insert(RecentDB, {ts=time() , lnk=link ,qnty= nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=sourceID, type="m", addinfo=nil}) 
			topper=true
			if refresh_requested == false then
				refresh_requested=true
				C_Timer.After(1, RecentRefreshList)
			end
		end
	end
end

function Recent:NEW_PET_ADDED(eventName, ...) ----------------------------------------------------------
	petID = ...
	if petID then
		local link = C_PetJournal_GetBattlePetLink(petID)

		Recent:NewDay()
		table.insert(RecentDB, {ts=time() , lnk=link ,qnty= nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=nil, type="p", addinfo=nil}) 
		topper=true
		if refresh_requested == false then
			refresh_requested=true
			C_Timer.After(1, RecentRefreshList)
		end
	end
end

function Recent:PLAYER_REGEN_DISABLED(eventName, ...) ----------------------------------------
	if RecentListFrame:IsShown() == true then
		popper=true
		RecentListFrame:Hide()
	end
end

function Recent:PLAYER_REGEN_ENABLED(eventName, ...) ----------------------------------------
	if RecentListFrame:IsShown() == false and popper==true then
		popper=false
		if topper==true then 
			RecentListFrame.ListScrollFrame.scrollBar:SetValue(1)
			topper=false
			RecentUpdateList()
		end 
		RecentListFrame:Show()
	end
end

function Recent:NewDay()  -------------------------------------------------------------------------------
	local max = table.getn(RecentDB)
	if max>0 and (Recent:sortDate(time()) ~= RecentDB[max].sd and RecentDB[max].type ~= 'h') then
		table.insert(RecentDB, {ts=RecentDB[max].ts , lnk='|cff9d9d9d|Hitem:4784::::::::60:::::|h[Lifeless Stone]|h|r' ,qnty= nil, sd=RecentDB[max].sd, loc=nil , iid=nil, type='h', addinfo=nil}) 
	end
end

function Recent:RecentCreateFilteredList(force) ---------------------------------------------------------
	refresh_requested=false
	local Junk = Recent.db.profile.showJunk
	local Mine = Recent.db.profile.showPartyLoot
	local Ach = Recent.db.profile.showAch
	local Mog = Recent.db.profile.showTrans
	local Pet = Recent.db.profile.showPets
	local Toy = Recent.db.profile.showToys
	local Mount = Recent.db.profile.showMounts
	local popup = false

	if (RecentDB) then								
		RecentFiltered={}											-- initialize or refilter FilterList
		local again=0
		for v, p in pairs(RecentDB) do  
			local itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(p.lnk)
			local one = ((p.type ~= "i" or  itemClassID== 2 or (itemClassID== 4 and not (itemEquipLoc == nil or itemEquipLoc == ''))) and Junk == false)  or Junk==true 
			local two = (Mine==true and p.player ~= PlayerID) or p.player==PlayerID or p.player==nil
			local three = (p.type == "a" and Ach== true) or p.type~= "a" 
			local four = (p.type == "m" and Mog== true) or p.type~= "m" 
			local five = (p.type == "p" and Pet== true) or p.type~= "p" 
			local six =  (p.type == "t" and Toy== true) or p.type~= "t" 
			local seven = (p.type == "n" and Mount== true) or p.type~= "n" 

			if one and two and three and four and five and six and seven then
				table.insert(RecentFiltered, {ts=p.ts , lnk=p.lnk ,qnty= p.qnty , sd=p.sd, loc=p.loc , iid=p.iid, type=p.type, addinfo=p.addinfo, player=p.player}) 

				if p.type == 'h' and again== 1 then 
					table.remove(RecentFiltered,table.getn(RecentFiltered))
				elseif p.type == 'h' then
					again=1
				else
					again=0
				end
				if (time() - p.ts) < 3 then popup=true end
			end
		end

		RecentListFrame.BottomLeftInset.SlotRequirementLabel:Hide(); RecentListFrame.BottomLeftInset.SlotLabel:Hide(); RecentListFrame.BottomLeftInset.WhenLabel:Hide(); 
		RecentListFrame.BottomLeftInset.WhereLabel:Hide(); RecentListFrame.BottomLeftInset.WhatLabel:Hide(); RecentListFrame.BottomLeftInset.WhatText:Hide();
		RecentListFrame.selectedSpellID=nil

		if popup==true and Recent.db.profile.popup==true then
			if not InCombatLockdown() then
				HybridScrollFrame_SetOffset(RecentListFrame.ListScrollFrame,1)
				RecentUpdateList()
				RecentListFrame:Show()
			else
				popper=true  
			end
		end
	end
end

function Recent:GetFullDate(weekday, month, day, year) ----------------------------------------------------------
	local weekdayName = CALENDAR_WEEKDAY_NAMES[weekday];
	local monthName = CALENDAR_FULLDATE_MONTH_NAMES[month];
	return weekdayName, monthName, day, year, month;   
end

function Recent:sameAsBefore(itemlink, userID) --------------------------------------------------------------------
	local max = table.getn(RecentDB)

	if max>1 then
		if RecentDB[max].type == 'h' then max = max -1 end
		if itemlink==RecentDB[max].lnk and userID==RecentDB[max].player then
			if (time() - RecentDB[max].ts) < 70 then
				return true
			else 
				return false
			end
		else
			return false
		end
	else
		return false
	end
end 

function Recent:localizedDate(...) ----------------------------------------------------------------------------------
	local weekday=date("%w",...)
	local m,d,y = string.match(date("%x",...), '(%d+)/(%d+)/(%d+)')
	return format(FULLDATE, Recent:GetFullDate(tonumber(weekday)+1, tonumber(m), tonumber(d), 2000+tonumber(y)))
end

function Recent:formattedTime(...) ----------------------------------------------------------------------------------
	local hour = tonumber(date("%H", ...))
	local minutes = tonumber(date("%M", ...))
	return (GameTime_GetFormattedTime(hour,minutes,true))
end

function Recent:sortDate(...) ---------------------------------------------------------------------------------------
	local m,d,y = string.match(date("%x",...), '(%d+)/(%d+)/(%d+)')
	return string.format('%s/%s/%s', y,m,d)
 end

function Recent:printtable() -----------------------------------------------------------------------------------------
    print('...................'.. table.getn(RecentDB))
    for v, p in pairs(RecentDB) do
       print(v, p.lnk, p.type, p.iid, p.sd )
    end
end

function Recent:openMount(link) --------------------------------------------------------------------------------------
	if not InCombatLockdown() then
		local mountID = 0
		local _, _, _, linkType, linkID = strsplit(":|H", link)
		if linkType == "item" then
			mountID = C_MountJournal.GetMountFromItem(tonumber(linkID))
		elseif linkType == "spell" then
			mountID = C_MountJournal.GetMountFromSpell(tonumber(linkID))
		end
		if not CollectionsJournal then
			CollectionsJournal_LoadUI()
		end
		SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_MOUNTS)
		MountJournal_SelectByMountID(mountID)
	end
end

function Recent:openPet(link) ---------------------------------------------------------------------------------------
   	if not InCombatLockdown() then
		local numPets, numOwned = C_PetJournal.GetNumPets()
		local out, speciesPOO
		local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",link)

		if linkType=="item" then
			local a, b, c, creatureID, d, e, f, g, h, j,k, displayID, speciesID = C_PetJournal.GetPetInfoByItemID(tonumber(linkID))
			speciesPOO=speciesID
		 else
			speciesPOO=tonumber(linkID)
		 end
		for i=1, numPets do
			local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, _, _, _, _, canBattle = C_PetJournal.GetPetInfoByIndex(i)
			if (speciesID == speciesPOO) then 
				out = petID 
				petIndex = i
				break;
			end
		end

		if petIndex then 
			if not CollectionsJournal then
				CollectionsJournal_LoadUI()
			end
			SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_PETS)
			PetJournalPetList_UpdateScrollPos(PetJournal.listScroll, petIndex)
			PetJournal_ShowPetCardByID(out)
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent:|r Pet is still to learn!", 1.0, 0.8, 0.2);
		end
	end
end

function Recent:setNew(vID) ------------------------------------------------------------------------------------------
	local indexOffset = (WardrobeCollectionFrame.ItemsCollectionFrame.PagingFrame:GetCurrentPage() - 1) * WardrobeCollectionFrame.ItemsCollectionFrame.PAGE_SIZE
	for i = 1, WardrobeCollectionFrame.ItemsCollectionFrame.PAGE_SIZE do
	   local model = WardrobeCollectionFrame.ItemsCollectionFrame.Models[i]
	   local index = i + indexOffset;
	   local visualInfo = WardrobeCollectionFrame.ItemsCollectionFrame.filteredVisualsList[index]
	   if visualInfo  then
		  if visualInfo.visualID==vID then 
			 model.NewString:Show()
			 model.NewGlow:Show()
			 break
		  end
	   end
	end
end

function Recent:setNewToy(vID) ------------------------------------------------------------------------------------------
	local _, _, _, _, linkID = strsplit(":|H", RDB[vID].lnk)    
	local itemID, _, _, _, _ = C_ToyBox_GetToyInfo(linkID)

	for i = 1, 18 do
		local button = ToyBox.iconsFrame["spellButton"..i];
		local itemIndex = (ToyBox.PagingFrame:GetCurrentPage() - 1) * 18 + button:GetID();
		local thisItemID = C_ToyBox_GetToyFromIndex(itemIndex)
		if thisItemID==itemID then
			button.new:Show()  
		end
	end
	SetCollectionsJournalShown(false, COLLECTIONS_JOURNAL_TAB_INDEX_TOYS)
end

function Recent_UpdateTooltip(self) ------------------------------------------------------------------------------------
	if (self:GetParent().index) then
		local bpet=false
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
		if (RDB[self:GetParent().index].type=="m") then
			local _, visualID, _, icon, _, itemLink, link = C_TransmogCollection.GetAppearanceSourceInfo(RDB[self:GetParent().index].iid)
			GameTooltip:SetHyperlink(itemLink)

		elseif (RDB[self:GetParent().index].type=="p") then
			local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[self:GetParent().index].lnk)
			if linkType == "battlepet" then		
				local _, speciesID, level, breedQuality, maxHealth, power, speed, battlePetID = strsplit(":", RDB[self:GetParent().index].lnk)
				BattlePetToolTip_Show(tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed), string.gsub(string.gsub(RDB[self:GetParent().index].lnk, "^(.*)%[", ""), "%](.*)$", ""))
				bpet=true
			else
				GameTooltip:SetHyperlink(RDB[self:GetParent().index].lnk)
			end
		else
			GameTooltip:SetHyperlink(RDB[self:GetParent().index].lnk)
		end

		local ltext=""
		local rtext=""

		if (RDB[self:GetParent().index].type=="n" or RDB[self:GetParent().index].type=="m" or RDB[self:GetParent().index].type=="p") then
			rtext = L["<Right Click> for preview"]
		end

		if RDB[self:GetParent().index].type=="t" then
			if not CollectionsJournal then
				_G.CollectionsJournal_LoadUI()
			end
			local page = ToyBox_FindPageForToyID(RDB[self:GetParent().index].iid)
			ltext = L["<Left Click> for collection"] .. " - |cFFff4040" .. string.format(PAGE_NUMBER, page) 
		end 

		if (RDB[self:GetParent().index].type=="a" or  RDB[self:GetParent().index].type=="n" or RDB[self:GetParent().index].type=="m" or RDB[self:GetParent().index].type=="p") then
			ltext = L["<Left Click> for collection"]
		end

		if (RDB[self:GetParent().index].type=="i") then
			itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[self:GetParent().index].lnk)
			if (itemEquipLoc == "INVTYPE_HEAD" or itemEquipLoc=="INVTYPE_SHOULDER" or itemEquipLoc=="INVTYPE_BODY" or itemEquipLoc=="INVTYPE_CHEST" or itemEquipLoc=="INVTYPE_ROBE" or itemEquipLoc=="INVTYPE_WAIST"  or itemEquipLoc=="INVTYPE_LEGS"  or itemEquipLoc=="INVTYPE_FEET"  or itemEquipLoc=="INVTYPE_WRIST"  or itemEquipLoc=="INVTYPE_HAND"  or itemEquipLoc=="INVTYPE_CLOAK"  or itemEquipLoc=="INVTYPE_WEAPON"  or itemEquipLoc=="INVTYPE_SHIELD"  or itemEquipLoc=="INVTYPE_2HWEAPON"  or itemEquipLoc=="INVTYPE_WEAPONMAINHAND"  or itemEquipLoc=="INVTYPE_WEAPONOFFHAND"  or itemEquipLoc=="INVTYPE_HOLDABLE" or itemEquipLoc=="INVTYPE_RANGED" or itemEquipLoc=="INVTYPE_THROWN" or itemEquipLoc=="INVTYPE_RANGEDRIGHT" or itemEquipLoc=="INVTYPE_TABARD") then
				rtext = L["<Right Click> for preview"]
			end	
		end

		if (rtext ~= "" or ltext ~= "" ) then
			if bpet then
				BattlePetTooltip:AddLine("__________________________________",0.5,0.5,0.5)		
				BattlePetTooltip:AddLine(" ",1,1,1)
				BattlePetTooltip:AddLine(ltext,0.8,0.8,0.8)
				BattlePetTooltip:AddLine(rtext,0.8,0.8,0.8)
			else
				GameTooltip:AddLine(" ",1,1,1)
				GameTooltip:AddDoubleLine(ltext ,rtext,0.8,0.8,0.8,0.8,0.8,0.8)
				GameTooltip:AddLine(" ",1,1,1)
				GameTooltip:Show()
			end
		end
	end
end
   
function RecentUpdateList()	--------------------------------------------------------------------------------------------------
	if RDB then
		local scrollFrame = RecentListFrame.ListScrollFrame
		local offset, _ = HybridScrollFrame_GetOffset(scrollFrame)
		local buttons = scrollFrame.buttons
		local prevdate=Recent:sortDate(time())
		local numDisplayedObjects = table.getn(RDB)
		local missinginfos=0

		for i=1, #buttons-1 do
			local itemName="x"
			local texture, itemName, itemClassID, itemSubClassID,itemRarity, itemLevel
			local button = buttons[i]
			local displayIndex = numDisplayedObjects - (i + offset)+1;

			if ( displayIndex > 0 ) then     
				local index = displayIndex
				local meta = nil
				button.factionIcon:SetShown(false)
				if index> 0 and (RDB[index].type == 'h') and index <= numDisplayedObjects then 

					button.name:SetFontObject("GameFontNormalLarge")
					button.name:SetJustifyH("CENTER")
					button.name:SetText('• ' .. Recent:localizedDate(RDB[index].ts) .. ' •')
					button.level:SetText(nil)
					button.icon:SetAlpha(0)
					prevdate=RDB[index].sd 
					button.selected = false
					button.selectedTexture:Hide()
					button.background:SetVertexColor(0, 0, 0, 0)
					button:SetHighlightTexture(nil)
					button.iconBorder:Hide()
					button.index = nil
					button.DragButton:SetEnabled(false)
					button.DragButton.ActiveTexture:Hide()
					button.name:SetTextColor(1,0.8,0.1,1)
					button.new:SetShown(false)
					button.new:SetText("")
					button.newGlow:SetShown(false)
					button:Show()

				elseif index <= numDisplayedObjects and index>0 then
					
					if time()-RDB[index].ts < 420 then	-- 7 mins
						button.newGlow:SetShown(true)
						button.newGlow:SetTexture("Interface\\AddOns\\Recent\\media\\new", "ADD")
						button.newGlow:SetVertexColor(1, 1, 0, 0.4) 
					else
						button.newGlow:SetShown(false)
					end


					if (RDB[index].type == "a" ) then			-- Achievement
						id, itemName, points, completed, month, day, year, description, flags, texture, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(RDB[index].iid)							
						itemClassID = nil
						itemSubClassID = nil
						itemRarity = 2
						button.name:SetTextColor(1,1,1,1)
						button.level:SetText("")
						if RDB[index].addinfo then 
							itemName=RDB[index].addinfo 
							meta=true
							button.name:SetTextColor(0.7,0.7,0.7,0.8)
						end

					elseif (RDB[index].type == "m") then									--Xmog
						local _, visualID, _, icon, _, itemLink, link = C_TransmogCollection.GetAppearanceSourceInfo(RDB[index].iid)
						itemName, _, _, _, _, _,_, _, _, _,	_, _, _, _, _,_, _ = GetItemInfo(itemLink)			
						texture=icon
						itemClassID = nil
						itemSubClassID = nil
						button.name:SetTextColor(1,0.5,1,1)
						button.level:SetText("")
						if (itemName==nil) then
							itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[index].lnk) 
							itemRarity = 1
							itemLevel = 0
							_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
						end

					elseif (RDB[index].type == "i" or RDB[index].type == "t" or RDB[index].type == "p" or RDB[index].type == "n") then			--Item
						
						local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[index].lnk)

						if linkType=="battlepet" then
							itemLevel,texture,itemName,itemRarity,missinginfos=Recent:getPet(RDB[index].lnk)	

							if (itemRarity==nil) then
								missinginfos = 1
								itemRarity = 1
								itemLevel = 0
								_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
							end

						else
							itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture,
							itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(RDB[index].lnk)

							if (itemRarity==nil) then
								itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[index].lnk) 
								itemRarity = 1
								itemLevel = 0
								_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
								missinginfos = 1
							else
								itemLevel, isPreview, baseILvl = GetDetailedItemLevelInfo(RDB[index].lnk)
							end
						end

						local red, green, blue, hex=GetItemQualityColor(itemRarity)	
						button.name:SetTextColor(red,green,blue,1)
						button.level:SetTextColor(red,green, blue,1)
						button.level:SetText("[".. itemLevel .. "]")

					end
					button.factionIcon:SetVertexColor(1, 1, 1, 1) 

					if (RDB[index].type == "p") then
						button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\pet")
						button.factionIcon:SetShown(true)

					elseif (RDB[index].type == "n") then
						button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\mount")
						button.factionIcon:SetShown(true)

					elseif (RDB[index].type == "t" ) then
						button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\toy")
						button.factionIcon:SetShown(true)

					elseif (RDB[index].type == "a" ) then
						button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\ACHIEVEMENT")
						button.factionIcon:SetShown(true)
						if meta==true then 
							button.factionIcon:SetVertexColor(0.3, 0.3, 0.8, 1) 
						end

					elseif (RDB[index].type == "m" ) then
						button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\mog")
						button.factionIcon:SetShown(true)

					else
					button.factionIcon:SetTexture("")
						button.factionIcon:SetShown(false)
					end
					
					local count=""
					if RDB[index].qnty ~= nil then
						if tonumber(RDB[index].qnty) >= 2 then count= "|cFFAAAAAA x "..RDB[index].qnty.. "|r"  end
					end
					
					if RDB[index].player and RDB[index].player ~= PlayerID then
						local rPerc, gPerc, bPerc, argbHex
						local name, class = strsplit("#", RDB[index].player)
						if class== nil then
							rPerc=0.5
							gPerc=0.5
							bPerc=0.5
						else
							rPerc, gPerc, bPerc, argbHex = GetClassColor(class)
						end
						button.new:SetTextColor(rPerc,gPerc,bPerc,1)
						button.new:SetShown(true)
						button.new:SetText('> '..name)
						button.background:SetVertexColor(1, 0.5, 0, 0.3)
						button.background:SetTexture("Interface\\AddOns\\Recent\\media\\reddot")
					else
						button.new:SetShown(false)
						button.new:SetText("")
						button.background:SetVertexColor(1, 1, 1, 0.6)
						button.background:SetTexture("Interface\\AddOns\\Recent\\media\\Tile")
					end		

					local nameText= itemName..count
					button.level:SetJustifyH("RIGHT")
					button.name:SetJustifyH("LEFT")
					button.name:SetText("   ".. nameText)
					button.icon:SetTexture(texture)
					button:SetHighlightTexture("Interface\\AddOns\\Recent\\media\\whitedot", "ADD") 
					button.index = index
					button.spellID = index

					button.active = active
					button:Show()
					
					if ( RecentListFrame.selectedSpellID == index ) then 
						button.selected = true;
						button.selectedTexture:Show()
						button.selectedTexture:SetTexture("Interface\\AddOns\\Recent\\media\\yellowdot", "ADD")	
						button.DragButton.ActiveTexture:Show()
					else
						button.selected = false
						button.selectedTexture:Hide()
						button.DragButton.ActiveTexture:Hide()
					end
					
					button:SetEnabled(true);
					button.DragButton:SetEnabled(true)
					button.additionalText = nil
					button.icon:SetDesaturated(false)
					button.icon:SetAlpha(1.0)
					button.name:SetFontObject("GameFontNormal")

					if ( button.showingTooltip ) then
						MountJournalMountButton_UpdateTooltip(button)
					end
				end
			else
				button:Hide()
			end
		end

		if (missinginfos==1) then 	C_Timer.After(1, function() RecentUpdateList() end)  end
		local totalHeight	 = (table.getn(RDB)) * 46
		HybridScrollFrame_Update(RecentListFrame.ListScrollFrame, totalHeight, RecentListFrame.ListScrollFrame:GetHeight())
	end
end

function Recent:RecentSetSelection(number) ----------------------------------------------------------------------------------
	if (number and table.getn(RDB)>0) then
		RecentListFrame.selectedSpellID=number
		if (RDB[number].loc) then 
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:Show()
			RecentListFrame.BottomLeftInset.WhenLabel:Show()
			RecentListFrame.BottomLeftInset.WhereLabel:Show()
			RecentListFrame.BottomLeftInset.SlotLabel:Show()
			RecentListFrame.BottomLeftInset.WhatLabel:Show()
			RecentListFrame.BottomLeftInset.WhatText:Show()
			RecentListFrame.BottomLeftInset.SlotLabel:SetText(Recent:formattedTime(RDB[number].ts) .. L[" "] .. " - " .. Recent:localizedDate(RDB[number].ts))
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:SetText(RDB[number].loc) 

			if (RDB[number].type=="i" or RDB[number].type=="t" or RDB[number].type=="n" or RDB[number].type=="p") then
				local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture,
				itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(RDB[number].lnk)		
				if itemRarity==nil then
					local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[number].lnk)
					local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(battlePetID); 
					if speciesID == nil then petName, petIcon, petType = C_PetJournal.GetPetInfoBySpeciesID(linkID) end
					if petType then
						itemEquipLoc="" 
						itemType=_G["TOOLTIP_BATTLE_PET"]	
						itemSubType=_G["BATTLE_PET_NAME_"..petType]
					end
				end
				RecentListFrame.BottomLeftInset.WhatText:SetText(itemType .. " > " .. (string.len(itemEquipLoc)>0 and itemSubType .. " > ".. _G[itemEquipLoc] or itemSubType))  

			elseif (RDB[number].type=="m") then
				RecentListFrame.BottomLeftInset.WhatText:SetText(L["Transmogrification"])
				
			elseif (RDB[number].type=="a") then
				RecentListFrame.BottomLeftInset.WhatText:SetText(BATTLE_PET_SOURCE_6)
			end

		else
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:Hide()
			RecentListFrame.BottomLeftInset.SlotLabel:Hide()
			RecentListFrame.BottomLeftInset.WhenLabel:Hide()
			RecentListFrame.BottomLeftInset.WhereLabel:Hide()
			RecentListFrame.BottomLeftInset.WhatLabel:Hide()
			RecentListFrame.BottomLeftInset.WhatText:Hide()
		end
		
		RecentListFrame.BottomLeftInset.WhenLabel:SetText(L[" When:"])
		RecentListFrame.BottomLeftInset.WhereLabel:SetText(L[" Where:"])
		RecentListFrame.BottomLeftInset.WhatLabel:SetText(L[" What:"])
		RecentUpdateList()
	end
end

function RecentListItem_OnClick(self, button) ----------------------------------------------------------------------------------
	Recent:RecentSetSelection(self.index)
end

function RecentMine_OnClick() --------------------------------------------------------------------------------------------------
	Recent.db.profile.showPartyLoot = RecentListFrame.RecentMineButton:GetChecked()
	Recent:RecentCreateFilteredList(true)
	RDB=RecentFiltered
	RecentUpdateList()
end

function RecentJunk_OnClick() ---------------------------------------------------------------------------------------------------
	Recent.db.profile.showJunk = RecentListFrame.RecentJunkButton:GetChecked()
	Recent:RecentCreateFilteredList(true)
	RDB=RecentFiltered
	RecentUpdateList()
end

function RecentListDragItem_OnClick(self, button) ----------------------------------------------------------------------------------
	if not InCombatLockdown() then
		if button == "LeftButton" then
			if not CollectionsJournal then
				_G.CollectionsJournal_LoadUI()
			end
		
			if RDB[self:GetParent().index].type == "p" then
				Recent:openPet(RDB[self:GetParent().index].lnk)

			elseif RDB[self:GetParent().index].type == "n" then
					Recent:openMount(RDB[self:GetParent().index].lnk)
		
			elseif RDB[self:GetParent().index].type == "t" then
				if (RDB[self:GetParent().index].iid ~= nil)  then

--					local _, _, _, _, linkID = strsplit(":|H", RDB[self:GetParent().index].lnk)    --
--					local itemID, _, _, _, _ = C_ToyBox.GetToyInfo(linkID) --
--					ToyBox.newToys[itemID] = true					--
					SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_TOYS)

--					local page = ToyBox_FindPageForToyID(RDB[self:GetParent().index].iid) --
--					if page then															--
--						ToyBox.PagingFrame:SetCurrentPage(page)						--
--						ToyBox.PagingFrame.currentPage=page					--
--						ToyBox.PagingFrame:Update();					--
--						C_Timer.After(0.25, function() Recent:setNewToy(self:GetParent().index) end)   --
--					end								--
				end
				
			elseif RDB[self:GetParent().index].type == "m" then
		
					local categoryID, visualID = C_TransmogCollection.GetAppearanceSourceInfo(RDB[self:GetParent().index].iid)
					local _, color, _, linkType, linkID = strsplit(":|H", RDB[self:GetParent().index].lnk)
					local mylink=linkType .. ":" .. tostring(linkID)                
					WardrobeCollectionFrame_OpenTransmogLink(mylink, nil)
					WardrobeCollectionFrame.ItemsCollectionFrame:RefreshVisualsList()
					WardrobeCollectionFrame.ItemsCollectionFrame:SelectVisual(visualID)
					WardrobeCollectionFrame.ItemsCollectionFrame:SetChosenVisualSource(visualID, RDB[self:GetParent().index].iid)
					C_Timer.After(0.25, function() Recent:setNew(visualID) end)  
		
			elseif RDB[self:GetParent().index].type == "a" then
		
					if not AchievementFrame then
						AchievementFrame_LoadUI()
					end
					ShowUIPanel(AchievementFrame)
					AchievementFrame_SelectAchievement(RDB[self:GetParent().index].iid)
			end
		
		elseif button == "RightButton" then

			if RDB[self:GetParent().index].type == "m" then
					local _, color, _, linkType, linkID = strsplit(":|H", RDB[self:GetParent().index].lnk);
					categoryID, visualID, canEnchant, icon, _, itemLink, transmogLink, _ = C_TransmogCollection.GetAppearanceSourceInfo(linkID)
					DressUpItemLink(itemLink)

			elseif RDB[self:GetParent().index].type == "i" or RDB[self:GetParent().index].type == "p" or RDB[self:GetParent().index].type == "n" then
					DressUpBattlePetLink(RDB[self:GetParent().index].lnk)
					DressUpMountLink(RDB[self:GetParent().index].lnk)
					DressUpItemLink(RDB[self:GetParent().index].lnk)	
			end
		end
		
		Recent:RecentSetSelection(self:GetParent().index)
	end
end

function Recent:GetOption(info) -------------------------------------------------------------------------------------------
	return self.db.profile[info[#info]]
end

function Recent:SetOption(info, newValue)
	self.db.profile[info[#info]] = newValue
	local r, g, b, a = getglobal(RecentListFrame.RecentMineButton:GetName() .. "Text"):GetTextColor()
	if newValue==true then
		RecentListFrame.RecentMineButton:Enable()
		getglobal(RecentListFrame.RecentMineButton:GetName() .. "Text"):SetTextColor(r,g,b,1) 
	else
		RecentListFrame.RecentMineButton:SetChecked(false)
		RecentListFrame.RecentMineButton:Disable()
		getglobal(RecentListFrame.RecentMineButton:GetName() .. "Text"):SetTextColor(r,g,b,0.4) 
		Recent.db.profile.showPartyLoot=false
		RecentMine_OnClick()
	end
end

function Recent:OnChatCommand(msg) -----------------------------------------------------------------------------------------
	if msg and (msg == 'clear') then
		RecentDB = table.wipe(RecentDB)
		RDB={}
		RecentFiltered={}
		RecentUpdateList()

		RecentListFrame.selectedSpellID=nil
		RecentListFrame.BottomLeftInset.SlotRequirementLabel:Hide()
		RecentListFrame.BottomLeftInset.SlotLabel:Hide()
		RecentListFrame.BottomLeftInset.WhenLabel:Hide()
		RecentListFrame.BottomLeftInset.WhereLabel:Hide()
		RecentListFrame.BottomLeftInset.WhatLabel:Hide()
		RecentListFrame.BottomLeftInset.WhatText:Hide()

		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent:|r List has been cleared...", 1.0, 0.8, 0.2);

	elseif msg and (msg == 'list') then 
		Recent:printtable()

	elseif msg and (msg=='togglemm') then	
		self.db.profile.minimap.hide = not self.db.profile.minimap.hide
		if self.db.profile.minimap.hide then
			icon:Hide("Recent")
		else
			icon:Show("Recent")
		end

	elseif msg and (msg == 'help') then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent|r - your very personal log", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("by |cFFEE4444Finnegan/Antonidas(EU)|r", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("---------------------------", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/togglemm - hide minimap button", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/clear - delete entire history", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/help - show this again", 1.0, 0.8, 0.2);
	else
	   	RecentListFrame:Show()
	end
end


function RecentFilterDropDown_OnLoad(self) ----------------------------------------------------------------------------------
	UIDropDownMenu_Initialize(self, RecentFilterDropDown_Initialize, "MENU");
end

function RecentFilterDropDown_Initialize(self, level) ----------------------------------------------------------------------------------
	local info = UIDropDownMenu_CreateInfo();
	info.keepShownOnClick = true;

	if level == 1 then
		info.text = L["Junk"]
		info.checked = Recent.db.profile.showJunk;
		info.func = function(_, _, _, value)
					Recent.db.profile.showJunk = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = ACHIEVEMENTS
		info.func = function(_, _, _, value)
					Recent.db.profile.showAch = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.checked = Recent.db.profile.showAch;
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = "Transmogs"
		info.func = function(_, _, _, value)
					Recent.db.profile.showTrans = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.checked = Recent.db.profile.showTrans;
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = PETS
		info.func = function(_, _, _, value)
					Recent.db.profile.showPets = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.checked = Recent.db.profile.showPets;
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = TOY
		info.func = function(_, _, _, value)
					Recent.db.profile.showToys = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.checked = Recent.db.profile.showToys;
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = MOUNTS
		info.func = function(_, _, _, value)
					Recent.db.profile.showMounts = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.checked = Recent.db.profile.showMounts;
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)


		info.text = "< • >"
		info.justifyH = "CENTER";
		info.notCheckable = true;
		info.isNotRadio = true;
		info.disabled = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = L["Partyloot"]
		info.func = function(_, _, _, value)
					Recent.db.profile.showPartyLoot = value;
					Recent:RecentCreateFilteredList(true);
					RDB=RecentFiltered;
					RecentUpdateList();
		end
		info.notCheckable = false;
		info.checked = Recent.db.profile.showPartyLoot;
		info.isNotRadio = true;
		info.disabled = not Recent.db.profile.logPartyLoot;
		UIDropDownMenu_AddButton(info, level)
	end
end

tinsert(UISpecialFrames, "RecentListFrame");
UIPanelWindows["RecentListFrame"] = nil;
